/* ====================================================================
 * Copyright (c) 2003 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <string.h>
#include <openssl/err.h>
#include <openssl/fips.h>
#include <openssl/des.h>
#include <openssl/opensslconf.h>

#ifdef OPENSSL_FIPS
static struct
    {
    DES_cblock key;
    unsigned char plaintext[8];
    unsigned char ciphertext[8];
    } tests[]=
	{
	{
	{ 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 },
	{ 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 },
	{ 0x8C,0xA6,0x4D,0xE9,0xC1,0xB1,0x23,0xA7 }
	},
	{
	{ 0xFE,0xDC,0xBA,0x98,0x76,0x54,0x32,0x10 },
	{ 0x01,0x23,0x45,0x67,0x89,0xAB,0xCD,0xEF },
	{ 0xED,0x39,0xD9,0x50,0xFA,0x74,0xBC,0xC4 },
	},
	};

static struct
    {
    DES_cblock key1;
    DES_cblock key2;
    unsigned char plaintext[8];
    unsigned char ciphertext[8];
    } tests2[]=
	{
	{
	{ 0x7c,0x4f,0x6e,0xf7,0xa2,0x04,0x16,0xec },
	{ 0x0b,0x6b,0x7c,0x9e,0x5e,0x19,0xa7,0xc4 },
	{ 0x06,0xa7,0xd8,0x79,0xaa,0xce,0x69,0xef },
	{ 0x4c,0x11,0x17,0x55,0xbf,0xc4,0x4e,0xfd }
	},
	{
	{ 0x5d,0x9e,0x01,0xd3,0x25,0xc7,0x3e,0x34 },
	{ 0x01,0x16,0x7c,0x85,0x23,0xdf,0xe0,0x68 },
	{ 0x9c,0x50,0x09,0x0f,0x5e,0x7d,0x69,0x7e },
	{ 0xd2,0x0b,0x18,0xdf,0xd9,0x0d,0x9e,0xff },
	}
	};

static struct
    {
    DES_cblock key1;
    DES_cblock key2;
    DES_cblock key3;
    unsigned char plaintext[8];
    unsigned char ciphertext[8];
    } tests3[]=
	{
	{
	{ 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 },
	{ 0xFE,0xDC,0xBA,0x98,0x76,0x54,0x32,0x10 },
	{ 0x12,0x34,0x56,0x78,0x9a,0xbc,0xde,0xf0 },
	{ 0x8f,0x8f,0xbf,0x9b,0x5d,0x48,0xb4,0x1c},
	{ 0x59,0x8c,0xe5,0xd3,0x6c,0xa2,0xea,0x1b},
	},
	{
	{ 0xDC,0xBA,0x98,0x76,0x54,0x32,0x10,0xFE },
	{ 0x01,0x23,0x45,0x67,0x89,0xAB,0xCD,0xEF },
	{ 0xED,0x39,0xD9,0x50,0xFA,0x74,0xBC,0xC4 },
	{ 0x01,0x23,0x45,0x67,0x89,0xAB,0xCD,0xEF },
	{ 0x11,0x25,0xb0,0x35,0xbe,0xa0,0x82,0x86 },
	},
	};

// CBC3 impl
static unsigned char cbc_key [8]={0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef};
static unsigned char cbc2_key[8]={0xf1,0xe0,0xd3,0xc2,0xb5,0xa4,0x97,0x86};
static unsigned char cbc3_key[8]={0xfe,0xdc,0xba,0x98,0x76,0x54,0x32,0x10};
static unsigned char cbc_iv  [8]={0xfe,0xdc,0xba,0x98,0x76,0x54,0x32,0x10};

static unsigned char cbc_data[40]={
    0x37,0x36,0x35,0x34,0x33,0x32,0x31,0x20,
    0x4E,0x6F,0x77,0x20,0x69,0x73,0x20,0x74,
    0x68,0x65,0x20,0x74,0x69,0x6D,0x65,0x20,
    0x66,0x6F,0x72,0x20,0x00,0x31,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    };
static unsigned char cbc3_ok[32]={
    0x3F,0xE3,0x01,0xC9,0x62,0xAC,0x01,0xD0,
    0x22,0x13,0x76,0x3C,0x1C,0xBD,0x4C,0xDC,
    0x79,0x96,0x57,0xC0,0x64,0xEC,0xF5,0xD4,
    0x1C,0x67,0x38,0x12,0xCF,0xDE,0x96,0x75};

void FIPS_corrupt_des()
    {
    tests[0].plaintext[0]++;
    }

int FIPS_selftest_des()
    {
    int n;

    /* Encrypt/decrypt with DES and compare to known answers */
    for(n=0 ; n < 2 ; ++n)
	{
	DES_key_schedule key;
	DES_cblock buf;

	DES_set_key(&tests[n].key,&key);
	DES_ecb_encrypt(&tests[n].plaintext,&buf,&key,1);
	if(memcmp(buf,tests[n].ciphertext,sizeof buf))
	    {
	    FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	    return 0;
	    }
	DES_ecb_encrypt(&tests[n].ciphertext,&buf,&key,0);
	if(memcmp(buf,tests[n].plaintext,sizeof buf))
	    {
	    FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	    return 0;
	    }
	}

    /* Encrypt/decrypt with 2-key 3DES and compare to known answers */
    for(n=0 ; n < 2 ; ++n)
	{
	DES_key_schedule key1, key2;
	unsigned char buf[8];

	DES_set_key(&tests2[n].key1,&key1);
	DES_set_key(&tests2[n].key2,&key2);
	DES_ecb2_encrypt(tests2[n].plaintext,buf,&key1,&key2,1);
	if(memcmp(buf,tests2[n].ciphertext,sizeof buf))
	    {
	    FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	    return 0;
	    }
	DES_ecb2_encrypt(tests2[n].ciphertext,buf,&key1,&key2,0);
	if(memcmp(buf,tests2[n].plaintext,sizeof buf))
	    {
	    FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	    return 0;
	    }
	}

    /* Encrypt/decrypt with 3DES and compare to known answers */
    for(n=0 ; n < 2 ; ++n)
	{
	DES_key_schedule key1, key2, key3;
	unsigned char buf[8];

	DES_set_key(&tests3[n].key1,&key1);
	DES_set_key(&tests3[n].key2,&key2);
	DES_set_key(&tests3[n].key3,&key3);
	DES_ecb3_encrypt(tests3[n].plaintext,buf,&key1,&key2,&key3,1);
	if(memcmp(buf,tests3[n].ciphertext,sizeof buf))
	    {
	    FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	    return 0;
	    }
	DES_ecb3_encrypt(tests3[n].ciphertext,buf,&key1,&key2,&key3,0);
	if(memcmp(buf,tests3[n].plaintext,sizeof buf))
	    {
	    FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	    return 0;
	    }
	}
}

/**
 * FIPS_selftest_des_cbc()
 * 
 * Description:
 *	KAT for TDES CBC mode, uses three keys.
 */
int FIPS_selftest_des_cbc()
{
    des_cblock iv3;
    des_key_schedule ks,ks2,ks3;
    int j;
    unsigned char cbc_in[40];
    unsigned char cbc_out[40];

    if ((j=DES_set_key_checked(&cbc_key,&ks)) != 0) {
	printf("Key error %d\n",j);
	FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	return (0);
    }
    if ((j=DES_set_key_checked(&cbc2_key,&ks2)) != 0) {
        printf("Key error %d\n",j);
	FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	return (0);
    }
    if ((j=DES_set_key_checked(&cbc3_key,&ks3)) != 0) {
        printf("Key error %d\n",j);
        FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	return (0);
    }

    memset(cbc_out, 0, sizeof(cbc_out));
    memset(cbc_in, 0, sizeof(cbc_out));

    int i = strlen((char *)cbc_data) + 1;

    memcpy(iv3, cbc_iv, sizeof(cbc_iv));

    des_ede3_cbc_encrypt(cbc_data,cbc_out,16L,ks,ks2,ks3,&iv3,
                 DES_ENCRYPT);
    des_ede3_cbc_encrypt(&(cbc_data[16]),&(cbc_out[16]),i-16,ks,ks2,ks3,
                 &iv3,DES_ENCRYPT);

    if (memcmp(cbc_out,cbc3_ok,
        (unsigned int)(strlen((char *)cbc_out)) ) != 0) {
        int n;

        printf("des_ede3_cbc_encrypt encrypt error\n");
        for(n=0 ; n < i ; ++n)
            printf(" %02x",cbc_out[n]);
        printf("\n");
        for(n=0 ; n < i ; ++n)
            printf(" %02x",cbc3_ok[n]);
        printf("\n");
	FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	return (0);
    }

    memcpy(iv3, cbc_iv, sizeof(cbc_iv));
    des_ede3_cbc_encrypt(cbc_out,cbc_in,i,ks,ks2,ks3,&iv3,DES_DECRYPT);

    if (memcmp(cbc_in,cbc_data,strlen((char *)cbc_data)+1) != 0) {
        int n;

        printf("des_ede3_cbc_encrypt decrypt error\n");
        for(n=0 ; n < i ; ++n)
            printf(" %02x",cbc_data[n]);
        printf("\n");
        for(n=0 ; n < i ; ++n)
            printf(" %02x",cbc_in[n]);
        printf("\n");
	FIPSerr(FIPS_F_FIPS_SELFTEST_DES,FIPS_R_SELFTEST_FAILED);
	return (0);
    }

    return (1);
}
#endif
